/**
 * Project Name:IDCardScanCaller
 * File Name:PreviewActivity.java
 * Package Name:com.intsig.idcardscancaller
 * Date:2016年3月15日下午2:14:46
 * Copyright (c) 2016, 上海合合信息 All Rights Reserved.
 *
 */

package com.intsig.dlcardscansdkcaller;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ArrayBlockingQueue;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Canvas;
import android.graphics.ImageFormat;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.RectF;
import android.graphics.Region;
import android.graphics.Path.Direction;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.CameraInfo;
import android.hardware.Camera.Size;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.Surface;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;

import com.intsig.dlcardscansdk.DLCardScanSDK;
import com.intsig.dlcardscansdk.ISCardScanActivity;
import com.intsig.dlcardscansdk.ResultData;

/**
 * ClassName:PreviewActivity <br/>
 * Function: TODO ADD FUNCTION. <br/>
 * Reason: TODO ADD REASON. <br/>
 * Date: 2016年3月15日 下午2:14:46 <br/>
 * 
 * @author guohua_xu
 */
public class PreviewActivity extends Activity implements Camera.PreviewCallback {
	private static final String TAG = "PreviewActivity";

	public static final String EXTRA_KEY_APP_KEY = "EXTRA_KEY_APP_KEY";

	private DetectThread mDetectThread = null;
	private Preview mPreview = null;
	private Camera mCamera = null;
	private int numberOfCameras;

	// The first rear facing camera
	private int defaultCameraId;

	private float mDensity = 2.0f;

	private DLCardScanSDK mIDCardScanSDK = null;

	private String mImageFolder = Environment.getExternalStorageDirectory()
			+ "/idcardscan/";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		mDensity = getResources().getDisplayMetrics().density;
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		// Hide the window title.
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);

		File file = new File(mImageFolder);
		if (!file.exists()) {
			file.mkdirs();
		}

		/*************************** set a SurfaceView as the content of our activity.******START ***********************/

		mPreview = new Preview(this);
		setContentView(mPreview);
		/*************************** Find the ID of the default camera******END ***********************/

		/*************************** Find the ID of the default camera******START ***********************/
		// Find the total number of cameras available
		numberOfCameras = Camera.getNumberOfCameras();
		// Find the ID of the default camera
		CameraInfo cameraInfo = new CameraInfo();
		for (int i = 0; i < numberOfCameras; i++) {
			Camera.getCameraInfo(i, cameraInfo);
			if (cameraInfo.facing == CameraInfo.CAMERA_FACING_BACK) {
				defaultCameraId = i;
			}
		}
		/*************************** Find the ID of the default camera******END ***********************/

		/*************************** Add mPreview Touch Listener******START ***********************/

		mPreview.setOnTouchListener(new View.OnTouchListener() {
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				if (mCamera != null) {
					mCamera.autoFocus(null);
				}
				return false;
			}
		});
		/*************************** Add mPreview Touch Listener******END ***********************/

		/*************************** init recog appkey ******START ***********************/
		mIDCardScanSDK = new DLCardScanSDK();
		Intent intent = getIntent();
		final String appkey = intent.getStringExtra(EXTRA_KEY_APP_KEY);
		new AsyncTask<Void, Void, Integer>() {
			@Override
			protected Integer doInBackground(Void... params) {
				int ret = mIDCardScanSDK.initCardScan(PreviewActivity.this,
						appkey);
				return ret;
			}

			@Override
			protected void onPostExecute(Integer result) {
				if (result != 0) {
					/**
					 * 101 包名错误, 授权APP_KEY与绑定的APP包名不匹配； 102
					 * appKey错误，传递的APP_KEY填写错误； 103 超过时间限制，授权的APP_KEY超出使用时间限制；
					 * 104 达到设备上限，授权的APP_KEY使用设备数量达到限制； 201
					 * 签名错误，授权的APP_KEY与绑定的APP签名不匹配； 202 其他错误，其他未知错误，比如初始化有问题；
					 * 203 服务器错误，第一次联网验证时，因服务器问题，没有验证通过； 204
					 * 网络错误，第一次联网验证时，没有网络连接，导致没有验证通过； 205
					 * 包名/签名错误，授权的APP_KEY与绑定的APP包名和签名都不匹配；
					 */
					new AlertDialog.Builder(PreviewActivity.this)
							.setMessage("Error " + result)
							.setNegativeButton(android.R.string.ok,
									new DialogInterface.OnClickListener() {
										@Override
										public void onClick(
												DialogInterface dialog,
												int which) {
											finish();
										}
									}).setCancelable(false).create().show();
				}
			}
		}.execute();
		/*************************** init recog appkey ******END ***********************/

	}

	boolean mNeedInitCameraInResume = false;

	@Override
	protected void onResume() {
		super.onResume();
		try {
			mCamera = Camera.open(defaultCameraId);// open the default camera
		} catch (Exception e) {
			e.printStackTrace();
			showFailedDialogAndFinish();
			return;
		}
		/********************************* preview是自定义的viewgroup 继承了surfaceview,将相机和surfaceview 通过holder关联 ***********************/
		mPreview.setCamera(mCamera);
		/********************************* 设置显示的图片和预览角度一致 ***********************/
		setDisplayOrientation();
		try {
			/********************************* 对surfaceview的PreviewCallback的 callback监听，回调onPreviewFrame ***********************/
			mCamera.setOneShotPreviewCallback(this);
		} catch (Exception e) {
			e.printStackTrace();

		}
		/*************************** 当按power键后,再回到程序,surface 不会调用created/changed,所以需要主动初始化相机参数******START ***********************/
		if (mNeedInitCameraInResume) {
			mPreview.surfaceCreated(mPreview.mHolder);
			mPreview.surfaceChanged(mPreview.mHolder, 0,
					mPreview.mSurfaceView.getWidth(),
					mPreview.mSurfaceView.getHeight());
			mHandler.sendEmptyMessageDelayed(100, 100);
		}
		mNeedInitCameraInResume = true;
		/********************************* END ***********************/

	}

	@Override
	protected void onPause() {
		super.onPause();
		// Because the Camera object is a shared resource, it's very
		// important to release it when the activity is paused.
		if (mCamera != null) {
			Camera camera = mCamera;
			mCamera = null;
			camera.setOneShotPreviewCallback(null);
			mPreview.setCamera(null);
			camera.release();
			camera = null;
		}
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		if (mIDCardScanSDK != null) {
			mIDCardScanSDK.release();
		}
		if (mDetectThread != null) {
			mDetectThread.stopRun();
		}
		mHandler.removeMessages(MSG_AUTO_FOCUS);
	}

	@Override
	public void onPreviewFrame(byte[] data, Camera camera) {
		Size size = camera.getParameters().getPreviewSize();
		if (mDetectThread == null) {
			mDetectThread = new DetectThread();
			mDetectThread.start();
			/********************************* 自动对焦的核心 启动handler 来进行循环对焦 ***********************/
			mHandler.sendEmptyMessageDelayed(MSG_AUTO_FOCUS, 100);
		}
		/********************************* 向预览线程队列中 加入预览的 data 分析是否ismatch ***********************/
		mDetectThread.addDetect(data, size.width, size.height);
	}

	private void showFailedDialogAndFinish() {
		new AlertDialog.Builder(this)
				.setMessage(R.string.fail_to_contect_camcard)
				.setCancelable(false)
				.setPositiveButton(android.R.string.ok,
						new DialogInterface.OnClickListener() {
							@Override
							public void onClick(DialogInterface dialog,
									int which) {
								finish();
							}
						}).create().show();
	}

	private void resumePreviewCallback() {
		if (mCamera != null) {
			mCamera.setOneShotPreviewCallback(this);
		}
	}

	/**
	 * 功能：将显示的照片和预览的方向一致
	 */
	private void setDisplayOrientation() {
		android.hardware.Camera.CameraInfo info = new android.hardware.Camera.CameraInfo();
		android.hardware.Camera.getCameraInfo(defaultCameraId, info);
		int rotation = getWindowManager().getDefaultDisplay().getRotation();
		int degrees = 0;
		switch (rotation) {
		case Surface.ROTATION_0:
			degrees = 0;
			break;
		case Surface.ROTATION_90:
			degrees = 90;
			break;
		case Surface.ROTATION_180:
			degrees = 180;
			break;
		case Surface.ROTATION_270:
			degrees = 270;
			break;
		}
		int result = (info.orientation - degrees + 360) % 360;
		Log.e(TAG, "setDisplayOrientation " + result + " info.orientation="
				+ info.orientation + " rotation=" + rotation);
		mCamera.setDisplayOrientation(result);
		/**
		 * 注释原因：因为FOCUS_MODE_CONTINUOUS_PICTURE 不一定兼容所有手机
		 * 小米4华为mate8对焦有问题，现在考虑用定时器来实现自动对焦
		 */

		// Camera.Parameters params = mCamera.getParameters();
		// String focusMode = Parameters.FOCUS_MODE_AUTO;
		// if (!TextUtils.equals("samsung", android.os.Build.MANUFACTURER)) {
		// focusMode = Parameters.FOCUS_MODE_CONTINUOUS_PICTURE;
		// }
		// Log.d("focusMode", focusMode);
		//
		// if (!isSupported(focusMode, params.getSupportedFocusModes())) {
		// // For some reasons, the driver does not support the current
		// // focus mode. Fall back to auto.
		// Log.d(" not isSupported", "not");
		//
		// if (isSupported(Parameters.FOCUS_MODE_AUTO,
		// params.getSupportedFocusModes())) {
		// focusMode = Parameters.FOCUS_MODE_AUTO;
		// } else {
		// focusMode = params.getFocusMode();
		// }
		// Log.d(" not isSupported", focusMode);
		//
		// }
		// params.setFocusMode(focusMode);
		// mCamera.setParameters(params);
		// if (!TextUtils.equals(focusMode,
		// Parameters.FOCUS_MODE_CONTINUOUS_PICTURE)) {
		// Log.d("FOCUS_MODE_CONTINUOUS_PICTURE", "not");
		// mHandler.sendEmptyMessageDelayed(MSG_AUTO_FOCUS, 2000);
		// }
	}

	public boolean isSupported(String value, List<String> supported) {
		return supported == null ? false : supported.indexOf(value) >= 0;
	}

	private static final int MSG_AUTO_FOCUS = 100;
	@SuppressLint("HandlerLeak")
	private Handler mHandler = new Handler() {
		public void handleMessage(Message msg) {
			if (msg.what == MSG_AUTO_FOCUS) {
				autoFocus();
			}
		};
	};

	private void autoFocus() {
		if (mCamera != null) {
			try {
				mCamera.autoFocus(focusCallback);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	boolean isFocus = false;

	/**
	 * 功能：对焦后的回调，每次返回bool值，如果对焦成功延时2秒对焦，如果失败继续对焦
	 */
	AutoFocusCallback focusCallback = new AutoFocusCallback() {

		@Override
		public void onAutoFocus(boolean success, Camera camera) {
			Log.d("lz", "success==" + success);

			if (success) {
				if (camera != null) {
					isFocus = true;
					mHandler.sendEmptyMessageDelayed(MSG_AUTO_FOCUS, 2000L);

				}
			} else {
				if (camera != null) {
					isFocus = false;
					mHandler.sendEmptyMessage(MSG_AUTO_FOCUS);
					// Log.d("lz", "isFocus==" + isFocus);

				}
			}

		}
	};

	boolean isVertical = false;

	/**
	 * @CN: 功能：根据当前屏幕的方向还有 宽高，还有证件的比例比如身份证高宽比 0.618来算出
	 *      预览框的位置和大小，可以更改此处来更改预览框的方向位置还有大小
	 * @EN :Function: according to the current screen orientation and high
	 *     width, and the proportion of documents such as identity card with
	 *     high aspect ratio the size and location of the 0.618 to calculate the
	 *     preview box, you can change here to change the direction of the
	 *     location and size of the preview box
	 * @param newWidth
	 * @param newHeight
	 * @return
	 */
	public Map<String, Float> getPositionWithArea(int newWidth, int newHeight,
			float scaleW, float scaleH) {
		float left, top, right, bottom;
		Map<String, Float> map = new HashMap<String, Float>();
		if (isVertical) {// vertical
			float dis = 1 / 16f;
			left = newWidth * dis;
			right = newWidth - left;
			top = 200f * scaleH;
			bottom = top + (newWidth - left - left) * 0.681f;
			/**
			 * 注解部分是：竖直模式居中，如果是竖直模式相对位置 则需要注意 不同分辨率手机的缩放比例
			 */
			// top = (newHeight - (newWidth - left - left) * 0.618f) / 2;
			// bottom = newHeight - top;
		} else {// horizental
			float dis = 1 / 8f;// 10
			left = newWidth * dis;
			right = newWidth - left;
			top = (newHeight - (newWidth - left - left) / 0.704f) / 2;
			bottom = newHeight - top;

			// top=400*scaleH;
			//
			// bottom=top+(newWidth - left - left) / 0.681f;

		}
		map.put("left", left);
		map.put("right", right);
		map.put("top", top);
		map.put("bottom", bottom);
		return map;

	}

	// thread to detect and recognize.
	/**
	 * 功能：将每一次预览的data 存入ArrayBlockingQueue 队列中，然后依次进行ismatch的验证，如果匹配就会就会进行进一步的识别
	 * 注意点： 1.其中 控制预览框的位置大小，需要
	 */
	private class DetectThread extends Thread {
		private ArrayBlockingQueue<byte[]> mPreviewQueue = new ArrayBlockingQueue<byte[]>(
				1);
		private int width;
		private int height;

		public void stopRun() {
			addDetect(new byte[] { 0 }, -1, -1);
		}

		@Override
		public void run() {
			try {
				while (true) {
					byte[] data = mPreviewQueue.take();// block here, if no data
														// in the queue.
					if (data.length == 1) {// quit the thread, if we got special
											// byte array put by stopRun().
						return;
					}
					float left, top, right, bottom;
					int newWidth = height;
					int newHeight = width;

					Map<String, Float> map = getPositionWithArea(newWidth,
							newHeight, 1, 1);
					left = map.get("left");
					right = map.get("right");
					top = map.get("top");
					bottom = map.get("bottom");

					// the (left, top, right, bottom) is base on preview image's
					// coordinate. that's different with ui coordinate.
					/********************************* 通过底册api 将预览的数据 还有证件的坐标位置 获取当前一帧证件的4个点坐标的数组 ***********************/

					int[] out = mIDCardScanSDK.detectBorder(data, width,
							height, (int) top, (int) (height - right),
							(int) bottom, (int) (height - left));
					// if activity is port mode then (x,y)->(preview.height-y,x)
					if (out != null) {// find border
						Log.d("DetectCard", "DetectCard >>>>>>>>>>>>> "
								+ Arrays.toString(out));
						for (int i = 0; i < 4; i++) {
							int tmp = out[0 + i * 2];
							out[0 + i * 2] = height - out[1 + i * 2];
							out[1 + i * 2] = tmp;
						}
						boolean match = isMatch((int) left, (int) top,
								(int) right, (int) bottom, out);
						/********************************* 实时画出预览 证件的虚拟边框，用来辅助 将证件 与预览框重合 更好识别 ***********************/
						mPreview.showBorder(out, match);

						/********************************* 当前预览帧的 证件四个点的坐标 和 预览框的证件4个点的坐标 校验，在一定范围内认定校验成功 ***********************/

						if (match && isFocus) { // get matched border
							ResultData result = mIDCardScanSDK.recognize(data,
									width, height, mImageFolder);

							if (result != null) {

								showResult(result);
								return;
							}
						}
					} else {// no find border, continue to preview;
						mPreview.showBorder(null, false);
					}
					// continue to preview;
					resumePreviewCallback();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		int continue_match_time = 0;

		/**
		 * 当前预览帧的 证件四个点的坐标 和 预览框的证件4个点的坐标 校验，在一定范围内认定校验成功
		 * 注意点：其中120是多次验证的值，没有其他理由校验比较稳定，这个值可以自己尝试改变
		 * 
		 * @param left
		 * @param top
		 * @param right
		 * @param bottom
		 * @param qua
		 * @return
		 */
		public boolean isMatch(int left, int top, int right, int bottom,
				int[] qua) {
			int dif = 120;
			int num = 0;

			if (Math.abs(left - qua[6]) < dif && Math.abs(top - qua[7]) < dif) {
				num++;
			}
			if (Math.abs(right - qua[0]) < dif && Math.abs(top - qua[1]) < dif) {
				num++;
			}
			if (Math.abs(right - qua[2]) < dif
					&& Math.abs(bottom - qua[3]) < dif) {
				num++;
			}
			if (Math.abs(left - qua[4]) < dif
					&& Math.abs(bottom - qua[5]) < dif) {
				num++;
			}
			Log.d(TAG, "inside " + Arrays.toString(qua) + " <>" + left + ", "
					+ top + ", " + right + ", " + bottom + "           " + num);
			if (num > 2) {
				continue_match_time++;
				if (continue_match_time >= 1)
					return true;
			} else {
				continue_match_time = 0;
			}
			return false;
		}

		public void addDetect(byte[] data, int width, int height) {
			if (mPreviewQueue.size() == 1) {
				mPreviewQueue.clear();
			}
			mPreviewQueue.add(data);
			this.width = width;
			this.height = height;
		}

		private void showResult(ResultData result) {
			Intent data = new Intent();
			if (!TextUtils.isEmpty(result.getOriImagePath())) {
				data.putExtra(ISCardScanActivity.EXTRA_KEY_RESULT_IMAGE,
						result.getOriImagePath());
			}
			if (!TextUtils.isEmpty(result.getTrimImagePath())) {
				data.putExtra(ISCardScanActivity.EXTRA_KEY_RESULT_IMAGE,
						result.getTrimImagePath());
			}
			data.putExtra(ISCardScanActivity.EXTRA_KEY_RESULT_DATA, result);
			setResult(RESULT_OK, data);
			finish();
		}
	}

	/**
	 * A simple wrapper around a Camera and a SurfaceView that renders a
	 * centered preview of the Camera to the surface. We need to center the
	 * SurfaceView because not all devices have cameras that support preview
	 * sizes at the same aspect ratio as the device's display.
	 */
	private class Preview extends ViewGroup implements SurfaceHolder.Callback {
		private final String TAG = "Preview";
		private SurfaceView mSurfaceView = null;
		private SurfaceHolder mHolder = null;
		private Size mPreviewSize = null;
		private List<Size> mSupportedPreviewSizes = null;
		private Camera mCamera = null;
		private DetectView mDetectView = null;
		private TextView mInfoView = null;
		private TextView mCopyRight = null;;

		public Preview(Context context) {
			super(context);
			/*********************************
			 * 自定义viewgrop上添加SurfaceView 然后对应的其他ui DetectView是自定义的预览框
			 * 
			 * ***********************/

			mSurfaceView = new SurfaceView(context);
			addView(mSurfaceView);

			mInfoView = new TextView(context);
			addView(mInfoView);

			mDetectView = new DetectView(context);
			addView(mDetectView);

			mCopyRight = new TextView(PreviewActivity.this);
			mCopyRight.setGravity(Gravity.CENTER);
			mCopyRight.setText(R.string.intsig_copyright);
			addView(mCopyRight);

			mHolder = mSurfaceView.getHolder();
			mHolder.addCallback(this);
		}

		public void setCamera(Camera camera) {
			mCamera = camera;
			if (mCamera != null) {
				mSupportedPreviewSizes = mCamera.getParameters()
						.getSupportedPreviewSizes();
				requestLayout();
			}
		}

		@Override
		protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
			// We purposely disregard child measurements because act as a
			// wrapper to a SurfaceView that centers the camera preview instead
			// of stretching it.
			final int width = resolveSize(getSuggestedMinimumWidth(),
					widthMeasureSpec);
			final int height = resolveSize(getSuggestedMinimumHeight(),
					heightMeasureSpec);
			setMeasuredDimension(width, height);

			if (mSupportedPreviewSizes != null) {
				mPreviewSize = getOptimalPreviewSize(mSupportedPreviewSizes,
						height, width);// 竖屏模式，寬高颠倒
			}
		}

		@Override
		protected void onLayout(boolean changed, int l, int t, int r, int b) {
			if (changed && getChildCount() > 0) {
				final View child = getChildAt(0);

				final int width = r - l;
				final int height = b - t;

				int previewWidth = width;
				int previewHeight = height;
				// if (mPreviewSize != null) {
				// previewWidth = mPreviewSize.height;
				// previewHeight = mPreviewSize.width;
				// }

				// Center the child SurfaceView within the parent.
				if (width * previewHeight > height * previewWidth) {
					final int scaledChildWidth = previewWidth * height
							/ previewHeight;
					child.layout((width - scaledChildWidth) / 2, 0,
							(width + scaledChildWidth) / 2, height);
					mDetectView.layout((width - scaledChildWidth) / 2, 0,
							(width + scaledChildWidth) / 2, height);
				} else {
					final int scaledChildHeight = previewHeight * width
							/ previewWidth;
					child.layout(0, (height - scaledChildHeight) / 2, width,
							(height + scaledChildHeight) / 2);
					mDetectView.layout(0, (height - scaledChildHeight) / 2,
							width, (height + scaledChildHeight) / 2);
				}
				getChildAt(1).layout(l, t, r, b);

				mCopyRight.layout(l, (int) (b - 48 * mDensity),
						(int) (r - 8 * mDensity), b);
			}
		}

		public void surfaceCreated(SurfaceHolder holder) {
			// The Surface has been created, acquire the camera and tell it
			// where to draw.
			try {
				if (mCamera != null) {
					mCamera.setPreviewDisplay(holder);
				}
			} catch (IOException exception) {
				Log.e(TAG, "IOException caused by setPreviewDisplay()",
						exception);
			}
		}

		public void surfaceDestroyed(SurfaceHolder holder) {
			// Surface will be destroyed when we return, so stop the preview.
			if (mCamera != null) {
				mCamera.stopPreview();
			}
		}

		private Size getOptimalPreviewSize(List<Size> sizes, int w, int h) {
			final double ASPECT_TOLERANCE = 0.1;
			double targetRatio = (double) w / h;
			if (sizes == null)
				return null;
			Size optimalSize = null;
			double minDiff = Double.MAX_VALUE;

			int targetHeight = h;

			// Try to find an size match aspect ratio and size
			for (Size size : sizes) {
				double ratio = (double) size.width / size.height;
				if (Math.abs(ratio - targetRatio) > ASPECT_TOLERANCE)
					continue;
				if (Math.abs(size.height - targetHeight) < minDiff&& Math.abs(ratio - 1.77f) < 0.02) {
					optimalSize = size;
					minDiff = Math.abs(size.height - targetHeight);
				}
			}

			// Cannot find the one match the aspect ratio, ignore the
			// requirement
			if (optimalSize == null) {
				minDiff = Double.MAX_VALUE;
				for (Size size : sizes) {
					if (Math.abs(size.height - targetHeight) < minDiff) {
						optimalSize = size;
						minDiff = Math.abs(size.height - targetHeight);
					}
				}
			}
			return optimalSize;
		}

		public void surfaceChanged(SurfaceHolder holder, int format, int w,
				int h) {
			if (mCamera != null) {
				// Now that the size is known, set up the camera parameters and
				// begin the preview.
				Camera.Parameters parameters = mCamera.getParameters();
				parameters.setPreviewSize(mPreviewSize.width,
						mPreviewSize.height);

				parameters.set("rotation", 270);

				parameters.setPreviewFormat(ImageFormat.NV21);
				requestLayout();
				mDetectView.setPreviewSize(mPreviewSize.width,
						mPreviewSize.height);
				mInfoView.setText("preview：" + mPreviewSize.width + ","
						+ mPreviewSize.height);
				mCamera.setParameters(parameters);
				mCamera.startPreview();
			}
		}

		public void showBorder(int[] border, boolean match) {
			mDetectView.showBorder(border, match);
		}
	}

	private int mColorNormal = 0xff2A7DF3;
	private int mColorMatch = 0xff01d2ff;

	/**
	 * the view show bank card border.
	 */
	@SuppressLint("InlinedApi")
	private class DetectView extends View {
		private Paint paint = null;
		private int[] border = null;
		private boolean match = false;
		private int previewWidth;
		private int previewHeight;
		// 蒙层位置路径
		Path mClipPath = new Path();
		RectF mClipRect = new RectF();
		float mRadius = 12;
		float cornerStrokeWidth = 8;
		float cornerSize = 30;// 4个角的大小

		public void showBorder(int[] border, boolean match) {
			this.border = border;
			this.match = match;
			postInvalidate();
		}

		public DetectView(Context context) {
			super(context);
			paint = new Paint();
			paint.setColor(0xffff0000);

			float density = getResources().getDisplayMetrics().density;
			mRadius = 12 * density;
			paint.setTextSize(16 * density);
			if (Build.VERSION.SDK_INT > Build.VERSION_CODES.HONEYCOMB) {
				setLayerType(View.LAYER_TYPE_SOFTWARE, null);
			}
			cornerStrokeWidth = 4 * density;
			cornerSize = 30 * density;

		}

		public void setPreviewSize(int width, int height) {
			this.previewWidth = width;
			this.previewHeight = height;
		}

		// 计算蒙层位置
		public void upateClipRegion(float scale, float scaleH) {
			float left, top, right, bottom;
			Map<String, Float> map = getPositionWithArea(getWidth(),
					getHeight(), scale, scaleH);
			left = map.get("left");
			right = map.get("right");
			top = map.get("top");
			bottom = map.get("bottom");
			mClipPath.reset();
			mClipRect.set(left, top, right, bottom);
			mClipPath.addRoundRect(mClipRect, mRadius, mRadius, Direction.CW);
		}

		@Override
		public void onDraw(Canvas c) {

			float scaleW;
			float scaleH;

			scaleW = getWidth() / (float) previewHeight;
			scaleH = getHeight() / (float) previewWidth;
			upateClipRegion(scaleW, scaleH);

			c.save();
			// 绘制 灰色蒙层
			c.clipPath(mClipPath, Region.Op.DIFFERENCE);
			c.drawColor(0xAA666666);
			c.restore();

			// 绘制 卡号, 过期日期等
			c.save();

			if (match) {// 设置颜色
				paint.setColor(mColorMatch);
			} else {
				paint.setColor(mColorNormal);
			}
			float len = cornerSize;
			float strokeWidth = cornerStrokeWidth;
			paint.setStrokeWidth(strokeWidth);
			c.drawLine(mClipRect.left, mClipRect.top + strokeWidth / 2,
					mClipRect.left + len + strokeWidth / 2, mClipRect.top
							+ strokeWidth / 2, paint);
			c.drawLine(mClipRect.left + strokeWidth / 2, mClipRect.top
					+ strokeWidth / 2, mClipRect.left + strokeWidth / 2,
					mClipRect.top + len + strokeWidth / 2, paint);
			// 右上
			c.drawLine(mClipRect.right - len - strokeWidth / 2, mClipRect.top
					+ strokeWidth / 2, mClipRect.right, mClipRect.top
					+ strokeWidth / 2, paint);
			c.drawLine(mClipRect.right - strokeWidth / 2, mClipRect.top
					+ strokeWidth / 2, mClipRect.right - strokeWidth / 2,
					mClipRect.top + len + strokeWidth / 2, paint);
			// 右下
			c.drawLine(mClipRect.right - len - strokeWidth / 2,
					mClipRect.bottom - strokeWidth / 2, mClipRect.right,
					mClipRect.bottom - strokeWidth / 2, paint);
			c.drawLine(mClipRect.right - strokeWidth / 2, mClipRect.bottom
					- len - strokeWidth / 2, mClipRect.right - strokeWidth / 2,
					mClipRect.bottom - strokeWidth / 2, paint);
			// 左下
			c.drawLine(mClipRect.left, mClipRect.bottom - strokeWidth / 2,
					mClipRect.left + len + strokeWidth / 2, mClipRect.bottom
							- strokeWidth / 2, paint);
			c.drawLine(mClipRect.left + strokeWidth / 2, mClipRect.bottom - len
					- strokeWidth / 2, mClipRect.left + strokeWidth / 2,
					mClipRect.bottom - strokeWidth / 2, paint);

			if (border != null) {
				paint.setStrokeWidth(3);
				float scale = getWidth() / (float) previewHeight;
				c.drawLine(border[0] * scale, border[1] * scale, border[2]
						* scale, border[3] * scale, paint);
				c.drawLine(border[2] * scale, border[3] * scale, border[4]
						* scale, border[5] * scale, paint);
				c.drawLine(border[4] * scale, border[5] * scale, border[6]
						* scale, border[7] * scale, paint);
				c.drawLine(border[6] * scale, border[7] * scale, border[0]
						* scale, border[1] * scale, paint);

			}

		}
	}
}
